/* -----------------------------------------------------------------------------
 * Copyright (c) 2014 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        13. January 2015
 * $Revision:    V1.00
 *
 * Project:      Flash Programming Functions for Toshiba TMPM07x Flash
 * --------------------------------------------------------------------------- */

/* History:
 *  Version 1.00
 *    Initial release
 */

#include "..\FlashOS.H"        // FlashOS Structures

typedef volatile unsigned long    vu32;
#define M32(adr) (*((volatile unsigned long  *)(adr)))

#define STACK_SIZE     64      // Stack Size
#define PAGE_SIZE      16      // Page Size  (4 words = 16 bytes)

/* Watchdog Timer (WD) */
typedef struct
{
  vu32 MOD;               /* WD Mode Register                             */
  vu32 CR;                /* WD Control Register                          */
} WD_TypeDef;

/* Flash Control (FC) */
typedef struct
{
  vu32 RESERVED0[4];
  vu32 PSCR;              /* Protect Security Control Register            */
  vu32 RESERVED1[3];
  vu32 PSSR;              /* Protect Security Status Register             */
  vu32 RESERVED2;
  vu32 FSR;               /* FC Status Register                           */
  vu32 SWPSR;             /* Memory Swap Status Register                  */
  vu32 PRGSR;             /* Program Status Register                      */
  vu32 MCR;               /* Mode Control Register                        */
  vu32 KYCR;              /* Flash Key Code Register                      */
  vu32 WEACR;             /* Flash Abort Control Register                 */
  vu32 RESERVED3[5];
  vu32 WAITCR;            /* Flash Wait Control Register                  */
} FC_TypeDef;

/* FC PSCR register definitions */
#define FCPSCR_PRT             (1UL << 4)

/* FC FSR register definitions */
#define FCFSR_FRDY             (1UL << 3)
#define FCFSR_BERR             (1UL << 2)
#define FCFSR_TOUT             (1UL << 1)

/* FC PRGSR register definitions */
#define FCPRGSR_RDY_BSY        (1UL << 0)

/* FC MCR register definitions */
#define FCMCR_MODE_RD          (0x0 << 3)
#define FCMCR_MODE_PG          (0x7 << 3)

/* FC KEYCR register definitions */
#define FCKYCR_KEYCODE_FCPSCR  (0xA74A9D23)
#define FCKYCR_KEYCODE_FCMCR   (0x596D432F)

#define FLASH_OFFSET           (0x3F800000)

#define WD_BASE       (0x400F2000UL)
#define FC_BASE       (0x41FFF000UL)

#define FC            ((FC_TypeDef *)  FC_BASE)
#define WD            ((WD_TypeDef *)  WD_BASE)


unsigned long base_adr;


/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int Init (unsigned long adr, unsigned long clk, unsigned long fnc) {

  // Disable Watchdog
  WD->MOD = 0x00;
  WD->CR  = 0xB1;

  base_adr = adr;

  return (0);
}


/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int UnInit (unsigned long fnc) {
  return (0);
}


/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseSector (unsigned long adr) {

  return (0);                                    // Automatic Erase during Program Cycle
}


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */

int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) {
  int ret = 0;                                   // set returncode to OK
  int len = (sz + 15) & ~15;                     // Make sure we are modulo 16

  while ((FC->FSR & FCFSR_FRDY) != FCFSR_FRDY);  // Wait until flash is ready

  // configure flash to program mode
  FC->KYCR = FCKYCR_KEYCODE_FCMCR;
  FC->MCR  = FCMCR_MODE_PG;

  while (len > 0) {
    // Write Data in junks of 4 * 4 words
    M32(adr + FLASH_OFFSET +  0) = *((unsigned long *)(buf +  0));
    M32(adr + FLASH_OFFSET +  4) = *((unsigned long *)(buf +  4));
    M32(adr + FLASH_OFFSET +  8) = *((unsigned long *)(buf +  8));
    M32(adr + FLASH_OFFSET + 12) = *((unsigned long *)(buf + 12));
    adr += 16;
    buf += 16;
    len -= 16;

    while ((FC->PRGSR & FCPRGSR_RDY_BSY) != FCPRGSR_RDY_BSY); // Wait until completed

    // check for errors
    if (FC->FSR & FCFSR_TOUT) {
      ret = 1;                                   // Finished without TimeOut
      break;
    }
  }

  // configure flash to read mode
  FC->KYCR = FCKYCR_KEYCODE_FCMCR;
  FC->MCR  = FCMCR_MODE_RD;

  return (ret);                                  // Finished without Errors
}
